/**
 * Copyright Copyright 2007-8 Simon Andrews
 *
 *    This file is part of StackMeasure.
 *
 *    StackMeasure is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    StackMeasure is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with StackMeasure; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
import ij.ImagePlus;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.filechooser.FileFilter;


public class FileListPanel extends JPanel implements ActionListener{

	private FileList list;
	private JButton addButton;
	private JButton deleteButton;
	private JTable table;
	private Vector listeners = new Vector();

	public FileListPanel (FileList list)  {
		this.list = list;
		setLayout(new BorderLayout());

		setBorder(BorderFactory.createLineBorder(Color.LIGHT_GRAY));

		add(new JLabel("Files to Process",JLabel.CENTER),BorderLayout.NORTH);

		
		table = new JTable(list.getTableModel());
		table.getColumnModel().getColumn(1).setCellRenderer(new ColorCellRenderer());

		add(new JScrollPane(table),BorderLayout.CENTER);
		
		JPanel buttonPanel = new JPanel();
		addButton = new JButton("Add files");
		addButton.addActionListener(this);
		addButton.setActionCommand("add");
		
		buttonPanel.add(addButton);
		
		deleteButton = new JButton("Delete");
		deleteButton.addActionListener(this);
		deleteButton.setActionCommand("delete");
		buttonPanel.add(deleteButton);
		
		
		add(buttonPanel,BorderLayout.SOUTH);
	}

	public void addListener (FilesListener l) {
		if (!listeners.contains(l)) {
			listeners.add(l);
		}
	}
	public void removeListener (FilesListener l) {
		if (listeners.contains(l)) {
			listeners.remove(l);
		}
	}
	
	private void filesOK () {
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((FilesListener)(e.nextElement())).filesOK();
		}
	}
	private void filesNotOK () {
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((FilesListener)(e.nextElement())).filesNotOK();
		}
	}

	
	public void actionPerformed(ActionEvent ae) {

		if (ae.getActionCommand().equals("add")) {
			JFileChooser chooser = new JFileChooser();
			chooser.setMultiSelectionEnabled(true);
			chooser.setFileFilter(new FileFilter() {
			
				public String getDescription() {
					return "Image Files";
				}
			
				public boolean accept(File f) {
					if (f.isDirectory() || f.getName().toLowerCase().endsWith(".tiff") || f.getName().toLowerCase().endsWith(".tif") || f.getName().toLowerCase().endsWith(".lsm")) {
						return true;
					}
					else {
						return false;
					}
				}
			
			});
			
			int result = chooser.showOpenDialog(this);
			if (result == JFileChooser.CANCEL_OPTION) return;

			File [] files = chooser.getSelectedFiles();

			for (int f=0;f<files.length;f++) {
				try {
					if (files[f].getName().toLowerCase().endsWith(".lsm")) {
						LSM_Reader lsmReader = new LSM_Reader();
						
						ImagePlus [] ips = lsmReader.open(files[f].getParent(),files[f].getName(),false,false);
						for (int i=0;i<ips.length;i++) {
							list.addFile(new StackMeasureFile(files[f],ips[i]));
						}
					}
					else {
						list.addFile(new StackMeasureFile(files[f]));
					}	
				}
				catch (IllegalArgumentException e) {
					JOptionPane.showMessageDialog(this,e.getMessage(),"Error adding image",JOptionPane.ERROR_MESSAGE);
				}
			}
		}
		else if (ae.getActionCommand().equals("delete")) {
			int [] rows = table.getSelectedRows();
			StackMeasureFile [] filesToDelete = new StackMeasureFile [rows.length];
			for (int r=0;r<rows.length;r++) {
				filesToDelete[r] = (StackMeasureFile)table.getModel().getValueAt(rows[r],0);
			}
			for (int f=0;f<filesToDelete.length;f++) {
				list.removeFile(filesToDelete[f]);
			}
		}
		
		
		// After the action check whether we're OK or not.
		if (list.getFileCount() > 0) {
			filesOK();
		}
		else {
			filesNotOK();
		}
		
	}
}
